/*
 * Ind_Connection.hpp
 *
 *  Created on: Jan 24, 2016
 *      Author: Joost Huizinga
 */

#ifndef IND_CONNECTION_HPP_
#define IND_CONNECTION_HPP_


/**
 * A connection class for neural networks.
 */
class Connection
{
public:
    /**
     * Constructor. Sets the source, target, weight and historic marker of this connections.
     */
    Connection(size_t source, size_t target, double weight):
        _source(source), _target(target), _weight(weight){

    }

    /**
     * Returns the weight of this connection.
     */
    const double& getWeight(){
        return _weight;
    }

    /**
     * Sets the weight of this connection.
     */
    void setWeight(double weight){
        _weight = weight;
    }

    /**
     * Returns the source of this connection.
     *
     * Note that the source is expressed as an index to be used with a map or vector.
     */
    const size_t& getSource(){
        return _source;
    }

    /**
     * Returns the target of this connection.
     *
     * Note that the target is expressed as an index to be used with a map or vector.
     */
    const size_t& getTarget(){
        return _target;
    }

protected:
    size_t _source;
    size_t _target;
    double _weight;
};



#endif /* IND_CONNECTION_HPP_ */
