/*
 * Ind_NeuralNetwork.hpp
 *
 *  Created on: Mar 6, 2015
 *      Author: Joost Huizinga
 */

#ifndef IND_NEURALNETWORK_HPP_
#define IND_NEURALNETWORK_HPP_

//Standard includes
#include <vector>
#include <fstream>

//Local includes
#include "Misc_Random.hpp"
#include "Ind_Neuron.hpp"
#include "Ind_Connection.hpp"

/**
 * A neural network designed to support recurrent neural networks.
 */
template<typename Neuron_t = Neuron, typename Connection_t = Connection>
class NeuralNetwork
{
public:
    /*******************************************
     ********* Assignment 2 functions **********
     *******************************************/

    /**
     * Initializes the network with the supplied number of neurons.
     *
     * Implement such that it initializes the network with the indicated number of neurons.
     */
    NeuralNetwork(size_t numberOfNeurons = 0){
        //TODO: YOUR CODE HERE
    }


    /**
     * Sets the minimum weight that a connection may get due to randomization or mutation.
     */
    void setMinWeight(double minWeight){
        _minWeight = minWeight;
    }

    /**
     * Sets the maximum weight that a connection may get due to randomization or mutation.
     */
    void setMaxWeight(double maxWeight){
        _maxWeight = maxWeight;
    }

    /**
     * Returns a reference to the vector of neurons of this network.
     */
    std::vector<Neuron_t>& getNeurons(){
        return _neurons;
    }

    /**
     * Returns a reference to the vector of neurons of this network.
     */
    std::vector<Connection_t>& getConnections(){
        return _connections;
    }

    /**
     * Sets the activation value of the indicated neuron.
     */
    void setValue(size_t neuronIndex, double value){
        _neurons[neuronIndex].setValue(value);
    }

    /**
     * Returns the activation value of the indicated neuron.
     */
    double getValue(size_t neuronIndex){
        if(neuronIndex >= _neurons.size()){
            std::cerr << "Ind_NeuralNetwork.hpp: Index out of bounds! Index: " << neuronIndex << " size: " << _neurons.size() << std::endl;
        }
        return _neurons[neuronIndex].getValue();
    }

    /**
     * Adds a neuron to this network.
     *
     * Implement such that it will add a neuron to the network.
     */
    void addNeuron(){
        //TODO: YOUR CODE HERE

    }

    /**
     * Adds a connection between the two indicated neurons.
     *
     * Implement such that it will add a connection between the two indicated neurons
     * with the indicated weight.
     *
     */
    void addConnection(size_t sourceIndex, size_t targetIndex, double weight = 0.0){
        //TODO: YOUR CODE HERE

    }


    /**
     * Performs one step of network activation.
     *
     * Implement such that it performs one step in the neural network.
     * One step in the neural network should include:
     * - Resetting the incoming values of neurons to 0.
     * - Updating the incoming values of all neurons by iterating over all their incoming
     *   connections, multiplying the activation value of the source neuron of those
     *   connections with the weight of those connections, and passing the sum to the
     *   target neuron with the updateIncoming method.
     * - For each neuron: propagate the incoming value to become the current activation
     *   of that neuron.
     *
     * Hint: this step may become more clear by first reading and implementing all
     * functions in the Neuron and Connection classes.
     */
    void step(){
        //TODO: YOUR CODE HERE

    }

    /**
     * Assigns random weights to each connection.
     *
     * Implement this function such that each connection is assigned, uniform randomly,
     * a number in [_minWeight, _maxWeight], and such that each neuron is assigned,
     * uniform randomly, a bias in [_minWeight, _maxWeight].
     */
    void randomize(){
        //TODO: YOUR CODE HERE

    }

    /**
     * Writes the current activation of the network to the output stream.
     *
     * Implement this function such that it writes the current activation of each
     * neuron to the supplied output stream. Separate each value with a space,
     * and write an end-of-line ("\n") once all values have been written.
     */
    void logActivation(std::ofstream& activationFile){
        //TODO: YOUR CODE HERE

    }

    /**
     * Resets all neurons in the network.
     *
     * This function is important for recurrent neurons because they may otherwise store
     * state.
     *
     * Implement such that it resets all neurons.
     */
    void reset(){
        //TODO: YOUR CODE HERE

    }

protected:
    //Vectors containing neurons and connections
    std::vector<Neuron_t> _neurons;
    std::vector<Connection_t> _connections;

    //Weight constraints
    double _minWeight;
    double _maxWeight;
};

/**
 * Convenience function for writing network connections to a file-stream.
 */
std::ostream& operator<<(std::ostream& is, NeuralNetwork<Neuron, Connection>& obj){
    std::vector<Neuron> neurons = obj.getNeurons();
    std::vector<Connection> connections = obj.getConnections();

    //Write neurons
    is << neurons.size() << " ";
    is << connections.size() << " ";
    for(size_t i=0; i<neurons.size(); ++i){
        is << neurons[i].getBias() << " ";
    }

    //Write connections
    for(size_t i=0; i<connections.size(); ++i){
        is << connections[i].getSource() << " " << connections[i].getTarget() << " "<< connections[i].getWeight()<< " ";
    }
    return is;
}



#endif /* IND_NEURALNETWORK_HPP_ */
